---
driver number: 0x90002
---

# Touch Panel

## Overview

The touch driver allows the process to interract with a touch panel.

## Command

  * ### Command number: `0`

    **Description**: Does the driver exist?

    **Argument 1**: unused

    **Argument 2**: unused

    **Returns**: Ok(()) if it exists, otherwise NODEVICE

  * ### Command number: `1`

    **Description**: Enable the single touch function of the panel. This will enable the touch panel if at least one app enables it.

    **Argument 1**: unused

    **Argument 2**: unused

    **Returns**: Ok(())

  * ### Command number: `2`

    **Description**: Disable the single touch function of panel. This will disable the touch panel if all apps disable it.

    **Argument 1**: unused

    **Argument 2**: unused

    **Returns**: Ok(())

  * ### Command number: `10`

    **Description**: Acknowledge that the multi touch buffer has been read. Multi touch events are reported by filling a previously `allows_readwrite` buffer. When receving a callback with the notification for a multi touch event, the app must acknowledge it before it can recevie another callback.

    **Argument 1**: unused

    **Argument 2**: unused

    **Returns**: Ok(())

  * ### Command number: `11`

    **Description**: Enable the multi touch function of the panel. This will enable the touch panel if at least one app enables it.

    **Argument 1**: unused

    **Argument 2**: unused

    **Returns**: Ok(())

  * ### Command number: `12`

    **Description**: Disable the multi touch function of panel. This will disable the touch panel if all apps disable it.

    **Argument 1**: unused

    **Argument 2**: unused

    **Returns**: Ok(())

  * ### Command number: `100`

    **Description**: Get the number of touch points available

    **Argument 1**: unused

    **Argument 2**: unused

    **Returns**: SUCCESS_U32 with U32 being the number of available touches

## Subscribe

  * ### Subscribe number: `0`

    **Description**: Subscribe to single touch.

    **Callback signature**: 
      - data1: status
      - data2: x (16 bit LE) | y (16 bit LE)
      - data3: pressure (16 bit LE) | area (16 bit LE) 

    **Returns**: Ok(()) if the subscribe was successful.

  * ### Subscribe number: `1`

    **Description**: Subscribe to gestures.

    **Callback signature**: 
      - data1: gesture

    **Returns**: Ok(()) if the subscribe was successful.

  * ### Subscribe number: `2`

    **Description**: Subscribe to multi touch events.

    **Callback signature**: 
      - data1: number of touch events
      - data2: number of dropped callback (ack sent too slow?)
      - data3: number of dropped touch events (shared buffer too small?)

    **Returns**: Ok(()) if the subscribe was successful.

## Allow ReadWrite

  * ### Allow number: `0`

    **Description**: Buffer to write multi touch events

    **Buffer format**:


    ```
    0         1           2                  4                  6           7             8         ...
    +---------+-----------+------------------+------------------+-----------+---------------+--------- ...
    | id (u8) | type (u8) | x (u16)          | y (u16)          | size (u8) | pressure (u8) |          ...
    +---------+-----------+------------------+------------------+-----------+---------------+--------- ...
    | Touch 0                                                                               | Touch 1  ...
    ```

    **Returns**: Ok(()) if the subscribe was successful, INVAL if the buffer's length is not a multiple of the color depth length. 

